
#include <SPI.h>
#include <SD.h>

#define THRESHOULD_SENSOR 100
#define ROLLING_TIME 3000
#define NUMBER_OF_TRIES 8
#define SHUTE_PIN A7



const int chipSelect = 4;


String inputString = "";         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete

String simString = "";
boolean simStringComplete = false; 
boolean smsIn = false;


int relays[4] { 22, 24, 26, 28 };
int SHUTE_PIN = A7;

char incoming_char=0;




void setup()
{

  randomSeed(analogRead(0));

  inputString.reserve(200);
  simString.reserve(200);

  for (int i=0; i<4; i++) {
    pinMode(relays[i], OUTPUT);
    digitalWrite(relays[i], HIGH);
  }

  pinMode(53, OUTPUT); digitalWrite(53, LOW);

  Serial.begin(9600); // for serial monitor
  Serial.println("Interactive RedBull Vending Machine...");
  Serial.println("by <luis.leao@gmail.com> and <afonso@yack.com.br>");

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");  

  Serial1.begin(2400); // for GSM shield
  SIM900power();  // turn on shield

  Serial1.print("ATE0\r"); delay(100);
  Serial1.print("AT\r"); delay(100);
  Serial1.print("AT+CMGF=1\r"); delay(100);
  Serial1.print("AT+CNMI=1,2,0,0,0\r"); delay(100);
  
  Serial.println("initiated");
}

void SIM900power()
// software equivalent of pressing the GSM shield "power" button
{
  pinMode(6, OUTPUT);
  digitalWrite(6, HIGH);
  delay(3000);
  digitalWrite(6, LOW);
  delay(7000);
  delay(10000);  // give time to log on to network.
}





void loop()
{
  serialEvent(); // check Arduino serial
  simEvent(); // check SIM900 serial
  
  if (stringComplete) { //received serial string
    Serial.print("> ");
    Serial.println(inputString);
    Serial1.print(inputString);
    Serial1.print('\r');
    
    inputString = "";
    stringComplete = false;
  }
  
  if (simStringComplete) {

    if (smsIn) {
      if (validateSms(simString)) {
        if (activateRelays()) {
          datalog("OK RELEASED < " + simString);
        } else {
          datalog("OK UNRELEASED < " + simString);
        }
      } else {
        datalog("ERRO < " + simString);
      }
      
      smsIn = false;
    }
    
    if (simString.indexOf("+CMT") >= 0) {
      Serial.println("RECEBEU SMS");
      smsIn = true;
      datalog(simString);
    }
    
    Serial.println(simString);
    simString = "";
    simStringComplete = false;
  }

}


void simEvent() {

  if (Serial1.available() > 0) {
    char inChar = (char)Serial1.read();
    if (inChar == '\r') {
      simStringComplete = true;
    } else if (inChar == '\n') {
    } else {
      simString += inChar;    
    }

  }

}


void serialEvent() {
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    // add it to the inputString:
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\r') {
      stringComplete = true;
    } else if (inChar == '\n') {
    } else {
      inputString += inChar;    
    }
  }
}


void datalog(String dataString) {

  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  File dataFile = SD.open("datalog.txt", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) {
    dataFile.println(dataString);
    dataFile.close();
  }
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog.txt");
  }  
}


boolean validateSms(String dataString) {
  int numero = dataString.substring(0, 4).toInt();
  int digito = dataString.substring(4, 5).toInt();
  
  int sum = dataString.substring(0, 1).toInt() +
  dataString.substring(1, 2).toInt() +
  dataString.substring(2, 3).toInt() +
  dataString.substring(3, 4).toInt();
  
  int check = sum % 10;

  //debug
  //Serial.print("NUMERO "); Serial.println(numero);
  //Serial.print("DIGITO "); Serial.println(digito);
  //Serial.print("CHECK "); Serial.println(check);

  return check == digito;
}



boolean bateu() {
  int in = analogRead(SHUTE_PIN);
  Serial.print("an "); Serial.println(in);
  return in >= THRESHOULD_SENSOR;
}



boolean activateRelays() {

  int tentativas = 0;
  boolean released = false;
  int current_release_millis;

  int relay_idx = random(0, 3);

  while (!released && tentativas < NUMBER_OF_TRIES) {
    tentativas++;
    //debug
    //Serial.print("tentativa "); Serial.print(tentativas); Serial.print(" rele "); Serial.println(relay_idx);
    
    digitalWrite(relays[relay_idx], LOW);
    current_release_millis = millis();
    int delay_time = millis() - current_release_millis;
    while (delay_time <= ROLLING_TIME) {
      delay_time = millis() - current_release_millis;
      Serial.print(millis()); Serial.print("\t"); Serial.print(delay_time); Serial.println();
      if (bateu()) {
        Serial.println("BATEU");
        released = true;
        break;
      }
    }
    digitalWrite(relays[relay_idx], HIGH);

    if (relay_idx == 3) {
      relay_idx = 0;
    } else {
      relay_idx++;
    }
  }
  return released;
}


